/**
* The contents of this file are subject to the Mozilla Public 
* License Version 1.1 (the "License"); you may not use this 
* file except in compliance with the License. You may obtain a
* copy of the License at http://www.mozilla.org/MPL/
*
* Software distributed under the License is distributed on an 
* "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express 
* or implied. See the License for the specific language 
* governing rights and limitations under the License.
*
* The Original Code is this file as it was released on
* March 17, 2005.
*
* The Initial Developer of the Original Code is Terry Yuen. 
* Portions created by Terry Yuen are Copyright (C) 2005.
* All Rights Reserved.
**/

/**
 * Dependencies:
 *  /tonguespeaker.js::TongueSpeaker()
 *  /tonguespeaker.js::TongueSpeaker2()
 */

var RiteOfTongue = {
  submenu: false,
  surlbar: false,
  speaker: null,

  startup: function() {
    window.removeEventListener("load", RiteOfTongue.startup, false);
    RiteOfTongue.loadPreferences();
    RiteOfTongue.hookToContextMenu();
  },

  loadPreferences: function() {
    const pref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("riteoftongue.");

    RiteOfTongue.submenu = pref.getBoolPref("show_in_submenu");
    RiteOfTongue.surlbar = pref.getBoolPref("show_in_urlbar");
    var wait = pref.getBoolPref("show_wait");
    var dict = pref.getBoolPref("show_add_to_dictionary");
    var select = pref.getBoolPref("selections_only");
    var key = pref.getCharPref("key");

    var tmp = /([^\/]+)\/(.+)/.exec(key);
    key = tmp[2];

    if(RiteOfTongue.speaker) {
      RiteOfTongue.speaker.destroy();
      RiteOfTongue.speaker = null;
    }

    if(tmp[1] == "yahoo") {
      RiteOfTongue.speaker = new TongueSpeaker(key, wait, RiteOfTongue.submenu, dict, select);
    } else {
      RiteOfTongue.speaker = new TongueSpeaker2(key, wait, RiteOfTongue.submenu, dict, select);
    }
  },

  hookToContextMenu: function() {
    var hook = function(m) {
      if(!m)return;
      m.addEventListener("popupshowing", RiteOfTongue.onPopupShowing, false);
      m.addEventListener("popuphiding", RiteOfTongue.onPopupHiding, false);
    };

    hook(document.getElementById("contentAreaContextMenu"));
    hook(document.getElementById("context-riteoftongue-popup"));
    if(RiteOfTongue.surlbar) {
      hook(document.getElementById("urlbar"));
      hook(document.getElementById("searchbar"));
    }
  },

  unhookFromContextMenu: function() {
    var unhook = function(m) {
      if(!m)return;
      m.removeEventListener("popupshowing", RiteOfTongue.onPopupShowing, false);
      m.removeEventListener("popuphiding", RiteOfTongue.onPopupHiding, false);
    };

    unhook(document.getElementById("contentAreaContextMenu"));
    unhook(document.getElementById("context-riteoftongue-popup"));
    if(RiteOfTongue.surlbar) {
      unhook(document.getElementById("urlbar"));
      unhook(document.getElementById("searchbar"));
    }
  },

  onPopupShowing: function(event) {
    const target = event.originalTarget;
    const target2 = event.explicitOriginalTarget;

    if(target.id == "contentAreaContextMenu") {
      document.getElementById("context-riteoftongue-separator").hidden = !RiteOfTongue.submenu || !gContextMenu.onTextInput;
      document.getElementById("context-riteoftongue-menu").hidden = !RiteOfTongue.submenu || !gContextMenu.onTextInput;
      if(!RiteOfTongue.submenu)RiteOfTongue.speaker.init(event);
    }

    else if(target.id == "context-riteoftongue-popup") {
      RiteOfTongue.speaker.init(event);
    }

    else if(target2.id == "urlbar" || target2.id == "searchbar") {
      RiteOfTongue.speaker.init(event);
    }
  },

  onPopupHiding: function(event) {
    const target = event.originalTarget;
    const target2 = event.explicitOriginalTarget;

    if(target.id == "contentAreaContextMenu") {
      if(!RiteOfTongue.submenu)RiteOfTongue.speaker.destroy();
    }

    else if(target.id == "context-riteoftongue-popup") {
      RiteOfTongue.speaker.destroy();
    }

    else if(target2.id == "urlbar" || target2.id == "searchbar") {
      RiteOfTongue.speaker.destroy();
    }
  },

  destroy: function() {
    window.removeEventListener("unload", RiteOfTongue.destroy, false);

    RiteOfTongue.unhookFromContextMenu();

    if(RiteOfTongue.speaker) {
      RiteOfTongue.speaker.destroy();
      RiteOfTongue.speaker = null;
    }
  }
};

window.addEventListener("load", RiteOfTongue.startup, false);
window.addEventListener("unload", RiteOfTongue.destroy, false);