/**
* The contents of this file are subject to the Mozilla Public 
* License Version 1.1 (the "License"); you may not use this 
* file except in compliance with the License. You may obtain a
* copy of the License at http://www.mozilla.org/MPL/
*
* Software distributed under the License is distributed on an 
* "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express 
* or implied. See the License for the specific language 
* governing rights and limitations under the License.
*
* The Original Code is this file as it was released on
* March 17, 2005.
*
* The Initial Developer of the Original Code is Terry Yuen. 
* Portions created by Terry Yuen are Copyright (C) 2005.
* All Rights Reserved.
**/

/**
 * Dependencies:
 *  /browseroverlay.xul (string bundle)
 */

var TongueDialect = {
  whitelist: null,
  listLoaded: false,

  inWhitelist: function(word) {
    if(!TongueDialect.listLoaded)TongueDialect.loadWhitelist();

    if(TongueDialect.whitelist) {
      word = word.toLowerCase();
      for(var i=0; i<TongueDialect.whitelist.length && TongueDialect.whitelist[i] <= word; i++) {
        if(TongueDialect.whitelist[i] == word) {
          return true;
        }
      }
    }
    return false;
  },

  addToWhitelist: function(word) {
    const prompter = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
    const pref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
    if(pref.getBoolPref("riteoftongue.ask_on_dictionary") && !prompter.confirm(window,
            document.getElementById("bundle_riteoftongue").getString("dialog.dictionary.title"),
            document.getElementById("bundle_riteoftongue").getFormattedString("dialog.dictionary.message", [word]))) {
      return;
    }

    if(!TongueDialect.listLoaded)TongueDialect.loadWhitelist();
    if(!TongueDialect.whitelist)TongueDialect.whitelist = new Array();

    word = word.toLowerCase();
    var i=0;
    for(; i<TongueDialect.whitelist.length; i++) {
      if(TongueDialect.whitelist[i] == word) {
        break;
      }
      else if(TongueDialect.whitelist[i] > word) {
        TongueDialect.whitelist.splice(i, 0, word);
        break;
      }
    }
    if(i == TongueDialect.whitelist.length)TongueDialect.whitelist.push(word);

    TongueDialect.saveWhitelist();
  },

  loadWhitelist: function() {
    TongueDialect.listLoaded = true;
    try {
      var file = TongueDialect.getWhitelistFile();

      if(file.exists() && file.isFile()) {
        var fis = Components.classes["@mozilla.org/network/file-input-stream;1"]
                    .createInstance(Components.interfaces.nsIFileInputStream)
                    .QueryInterface(Components.interfaces.nsILineInputStream);
        fis.init(file, 0x01, 0400, 0);

        var line = {};
        var arr = [];
        while(fis.readLine(line)) {
          arr.push(line.value);
        }
        if(line.value)arr.push(line.value);

        fis.close();
        if(arr.length > 0)TongueDialect.whitelist = arr;
      }
    } catch(e) {
      dump("\nRiteOfTongue: Error while loading dictionary.\n" + e);  //io error
    }
  },

  saveWhitelist: function() {
    try {
      var file = TongueDialect.getWhitelistFile();

      var data = "";
      if(TongueDialect.whitelist) {
        for(var i=0; i<TongueDialect.whitelist.length; i++) {
          data += TongueDialect.whitelist[i] + "\n";
        }
      }

      var fos = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(Components.interfaces.nsIFileOutputStream);
      fos.init(file, 0x2a, 0600, 0); // write|create|truncate
      fos.write(data, data.length);
      fos.close();
    } catch(e) {
      dump("\nRiteOfTongue: Error while saving dictionary.\n" + e);  //io error
    }
  },

  getWhitelistFile: function() {
    var file = null;
    try {
      file = Components.classes["@mozilla.org/preferences-service;1"]
               .getService(Components.interfaces.nsIPrefBranch)
               .getComplexValue("riteoftongue.dictionary.path", Components.interfaces.nsILocalFile);
    } catch(e) {}
    
    if(!file) {
      file = Components.classes["@mozilla.org/file/directory_service;1"]
               .getService(Components.interfaces.nsIProperties)
               .get("ProfD", Components.interfaces.nsIFile);
      file.append("dictionary.dic");
    }

    return file;
  }
};